import os
import gzip
import json
import shutil

from datetime import datetime

with open(os.getcwd()+os.path.sep+"config.json", "r", encoding='utf8') as file:
    json_contents = json.loads(file.read())
    log_size = json_contents["logging"]["size"]
    log_folder = json_contents["logging"]["location"]
    file.close()

# Check latest log size
def checkSize(debug=False):

    global log_folder

    if debug:
        log_file = "debug.log"
    else:
        log_file = "latest.log"

    try:
        os.makedirs(log_folder, exist_ok=True)
        log = os.stat(os.path.join(log_folder, log_file))
        if (log.st_size / 1024) > log_size:
            with open(os.path.join(log_folder, log_file), 'rb') as f_in:
                with gzip.open(os.path.join(log_folder, f'{datetime.now().strftime("%d.%m.%Y_%H:%M:%S")}.log.gz'), 'wb') as f_out:
                    shutil.copyfileobj(f_in, f_out)
                    print(f'Copied {os.path.join(log_folder, datetime.now().strftime("%d.%m.%Y_%H:%M:%S"))}.log.gz')
            open(os.path.join(log_folder, log_file), 'w').close()
    except FileNotFoundError:
        print(f'Log file {os.path.join(log_folder, log_file)} does not exist')
        pass

# Append string to log
def logAppend(message, debug=False):

    global log_folder

    message_formatted = f'[{datetime.now().strftime("%d.%m.%Y")}] [{datetime.now().strftime("%H:%M:%S")}] {message}'
    checkSize(debug=debug)

    if debug:
        log_file = "debug.log"
    else:
        log_file = "latest.log"

    log = open(os.path.join(log_folder, log_file), 'a')
    log.write(f'{message_formatted}\n')
    log.close()

# Print to stdout and then to log
def logWrite(message, debug=False):
    # save to log file and rotation is to be done
    logAppend(f'{message}', debug=debug)
    print(f"{message}", flush=True)